@@ -65,7 +65,7 @@
 /* Buffers received packets of type APPLICATION DATA,
  * HANDSHAKE DATA and HEARTBEAT.
  */
-void
+int
 _gnutls_record_buffer_put(gnutls_session_t session,
 			  content_type_t type, uint64 * seq,
 			  mbuffer_st * bufel)
@@ -78,7 +78,7 @@ _gnutls_record_buffer_put(gnutls_session_t session,
 	_gnutls_buffers_log("BUF[REC]: Inserted %d bytes of Data(%d)\n",
 			    (int) bufel->msg.size, (int) type);
 
-	return;
+	return 0;
 }
 
 /**
@@ -218,7 +218,7 @@ _gnutls_dgram_read(gnutls_session_t session, mbuffer_st ** bufel,
 		gettime(&t1);
 	}
 
-	*bufel = _mbuffer_alloc(0, max_size);
+	*bufel = _mbuffer_alloc_align16(max_size, get_total_headers(session));
 	if (*bufel == NULL)
 		return gnutls_assert_val(GNUTLS_E_MEMORY_ERROR);
 
@@ -286,7 +286,7 @@ _gnutls_stream_read(gnutls_session_t session, mbuffer_st ** bufel,
 
 	session->internals.direction = 0;
 
-	*bufel = _mbuffer_alloc(0, MAX(max_size, size));
+	*bufel = _mbuffer_alloc_align16(MAX(max_size, size), get_total_headers(session));
 	if (!*bufel) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
@@ -666,7 +666,7 @@ ssize_t _gnutls_io_write_flush(gnutls_session_t session)
  */
 int _gnutls_io_check_recv(gnutls_session_t session, unsigned int ms)
 {
-	gnutls_transport_ptr_t fd = session->internals.transport_recv_ptr;
+	gnutls_transport_ptr_t fd = session->internals.transport_send_ptr;
 	int ret = 0, err;
 
 	if (unlikely
@@ -924,7 +924,7 @@ static int merge_handshake_packet(gnutls_session_t session,
 	if (exists == 0)
 		pos = session->internals.handshake_recv_buffer_size;
 
-	if (pos >= MAX_HANDSHAKE_MSGS)
+	if (pos > MAX_HANDSHAKE_MSGS)
 		return
 		    gnutls_assert_val(GNUTLS_E_TOO_MANY_HANDSHAKE_PACKETS);
 
@@ -1310,7 +1310,7 @@ _gnutls_handshake_io_recv_int(gnutls_session_t session,
 			return gnutls_assert_val(ret);
 	}
 
-	if (htype != (unsigned) -1) {
+	if (htype != (gnutls_handshake_description_t) -1) {
 		ret = handshake_remaining_time(session);
 		if (ret < 0)
 			return gnutls_assert_val(ret);
